"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sanitizeConfig = void 0;
const node_crypto_1 = require("node:crypto");
const DEFAULT_TLS_EXPIRY_REMINDER_DAYS = 30;
const DEFAULT_STATUS_NOTIFICATION = '0 6 * * *';
function sanitizeConfig(config) {
    const { certificate, notifications = [], version } = config;
    const sanitizedConfigWithoutVersion = {
        ...config,
        certificate: sanitizeCertificate(certificate),
        notifications,
        'status-notification': config['status-notification'] || DEFAULT_STATUS_NOTIFICATION,
    };
    return {
        ...sanitizedConfigWithoutVersion,
        version: version || md5Hash(sanitizedConfigWithoutVersion),
    };
}
exports.sanitizeConfig = sanitizeConfig;
function md5Hash(data) {
    return (0, node_crypto_1.createHash)('md5').update(JSON.stringify(data)).digest('hex');
}
function sanitizeCertificate(certificate) {
    if (!certificate) {
        return certificate;
    }
    return {
        ...certificate,
        reminder: certificate.reminder || DEFAULT_TLS_EXPIRY_REMINDER_DAYS,
    };
}
//# sourceMappingURL=sanitize.js.map